%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DCI_FORMAT0 Prints & returns contents of Format-0 DCI frame
% Requires: Bits of DCI Format-0 frame
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [format01, frequency_hopping, rb_assignment, mcs, new_data_indicator, tpc_command, cyclic_shift, cqi_request, multi_cluster_flag, rbstart, lcrb, crnti, crnti_code, startsc, endsc] = dci_format0(data)
    nfft_used = 600;
    format01 = data(1);
    frequency_hopping = data(2);
    rb_assignment = data(3:13);    
    mcs = data(14:18);
    new_data_indicator = data(19);
    tpc_command = data(20:21);
    cyclic_shift = data(22:24);
    cqi_request = data(25);
    multi_cluster_flag = data(26);
    crnti = data(end-15:end);
    crnti_code = bin2dec(int2str(crnti.'));
    
    fprintf(' Format = %d\n Frequency Hopping = %d\n RB-Assignment = [%d %d %d %d %d %d %d %d %d %d %d]\n MCS = [%d %d %d %d %d]\n New Data Indicator = %d\n TPC Command = [%d %d]\n Cyclic Shift = [%d %d %d]\n CQI request = %d\n Multi-Cluster-Flag = %d\n CRNTI = [%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d] = %d\n', format01, frequency_hopping, rb_assignment, mcs, new_data_indicator, tpc_command, cyclic_shift, cqi_request, multi_cluster_flag, crnti, bin2dec(int2str(crnti.')));    
    
    % Process RB Assignment
    nulrb = 50;
    %riv = bin2dec(int2str(rb_assignment(end:-1:1).'));
    riv = bin2dec(int2str(rb_assignment(1:1:end).'));
    rbstart = mod(riv, nulrb);
    lcrb = floor(riv/nulrb)+1;   
    startsc = rbstart*12-nfft_used/2;
    endsc = rbstart*12-nfft_used/2 + lcrb*12;
        
    lcrb_1 = nulrb + 1 - floor(riv/nulrb);   
    rbstart_1 = nulrb - 1 - mod(riv, nulrb);
    startsc_1 = rbstart_1*12-nfft_used/2;
    endsc_1 = rbstart_1*12-nfft_used/2 + lcrb_1*12;
    
    if(~(startsc >= -nfft_used/2 && endsc <= nfft_used/2))
        startsc = startsc_1;
        endsc   = endsc_1;
        lcrb    = lcrb_1;
        rbstart = rbstart_1;
    end
    
    % Correction?
    if(~(startsc<0 && endsc > 0))
        if(startsc<0)
            endsc = endsc-1;
        else
            startsc = startsc+1;
        end
    end   
    
    fprintf(' Actual subcarriers occupied by CRNTI=%d on uplink: [%d to %d]\n', crnti_code, rbstart*12-nfft_used/2, rbstart*12-nfft_used/2 + lcrb*12);
    
end
