%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DCI_FORMAT2 Prints & returns contents of Format-2 DCI frame
% Requires: Bits of DCI Format-2 frame
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [resource_alloc_hdr, rb_assignment, tpc, harq, tbcw_flag, tb1_mcs, tb1_ndi, tb1_rv, tb2_mcs, tb2_ndi, tb2_rv, crnti, crnti_code, precoding] = dci_format2(data)
    nfft_used = 600;
    ndlrb = 50;
    P = 3; % Resource block group size
    Nrbg = ceil(ndlrb/P); 
    resource_alloc_hdr = data(1); % 1 bit
    if(resource_alloc_hdr == 0)        
        rb_assignment = data(2:1+Nrbg); % Nrbg 17 bits
    else
        fprintf(2, 'Error: Not implemented RBG format 1');
    end
    tpc = data(Nrbg+2:Nrbg+3); % 2 bits
    harq = data(Nrbg+4:Nrbg+6); % 3 bits
    tbcw_flag = data(Nrbg+7); % 1 bit
    tb1_mcs = data(Nrbg+8:Nrbg+12); % 5 bits
    tb1_ndi = data(Nrbg+13); % 1 bit
    tb1_rv = data(Nrbg+14:Nrbg+15); % 2 bits
    tb2_mcs = data(Nrbg+16:Nrbg+20); % 5 bits
    tb2_ndi = data(Nrbg+21); % 1 bit
    tb2_rv = data(Nrbg+22:Nrbg+23); % 2 bits    
    precoding = data(Nrbg+24:Nrbg+26); % 3 bits    
    crnti = reshape(data(end-15:end), [], 1); % 16 bits
    crnti_code = bin2dec(int2str(crnti.'));
    rb_code = bin2dec(int2str(rb_assignment.'));
    nscdl = sum(rb_assignment)*12*P;
    
    last_start = 0;    
    cur_end = 0;
    sclist = [];
    for ii=1:length(rb_assignment)
        if(rb_assignment(ii)==1)
            if(last_start == 0)
                last_start = -300+36*(ii-1);
                cur_end = -300+36*ii-1;
            else
                cur_end = -300+36*ii-1;                
            end
        else
            if(last_start ~= 0)                
                sclist = [sclist last_start cur_end];
                last_start = 0;
                cur_end = 0;
            end
        end
    end
    
    if(last_start ~= 0)
        sclist = [sclist last_start cur_end];        
    end
    sclist(sclist>=0) = sclist(sclist>=0)+1;
    
    fprintf(' Resource Alloc Hdr = %d\n RB-Assignment = [%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d]\n TPC Command = [%d %d]\n HARQ = [%d %d %d]\n TBCW_flag = %d\n TB1-MCS = [%d %d %d %d %d]\n TB1-NDI = %d\n TB1-RV = [%d %d]\n TB2-MCS = [%d %d %d %d %d]\n TB2-NDI = %d\n TB2-RV = [%d %d]\n CRNTI = [%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d] = %d\n Precoding = [%d %d %d]\n', resource_alloc_hdr, rb_assignment, tpc, harq, tbcw_flag, tb1_mcs, tb1_ndi, tb1_rv, tb2_mcs, tb2_ndi, tb2_rv, crnti, crnti_code, precoding);    
    fprintf(' Actual subcarriers meant for CRNTI=%d on downlink: ', crnti_code);
    fprintf('[%d -> %d] ', sclist);
    fprintf('\n');
    
end
