%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DCI_FORMAT2a Prints & returns contents of Format-2a DCI frame
% Requires: Bits of DCI Format-2a frame
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [resource_alloc_hdr, rb_bitmap, tpc, harq, tbcw_flag, tb1_mcs, tb1_ndi, tb1_rv, tb2_mcs, tb2_ndi, tb2_rv, crnti, crnti_code,rb_subset] = dci_format2a(data)
    nfft_used = 600;
    ndlrb = 50;
    P = 3; % Resource block group size
    Nrbg = ceil(ndlrb/P); 
    resource_alloc_hdr = data(1); % 1 bit
    rb_bitmap=zeros(1,ndlrb);
    subset_sizes=[18,17,15];
    rb_subset=0;
    if(resource_alloc_hdr == 0)        
        rb_assignment = data(2:1+Nrbg); % Nrbg 17 bits
        rb_bitmap(1:3:ndlrb)=rb_assignment;
        rb_bitmap(2:3:ndlrb)=rb_assignment;
        rb_bitmap(3:3:ndlrb)=rb_assignment(1:end-1);
    else
        rb_subset = bin2dec(int2str(data(2:3).'));
        rb_shift = data(4);
        rb_assignment = data(5:1+Nrbg);
        if(rb_subset<3)
            ind = (0:13) + (subset_sizes(rb_subset+1)-14)*rb_shift;
            mapped_ind=(P^2)*floor(ind/P)+(P*rb_subset)+mod(ind,P);
            rb_bitmap(mapped_ind+1)=rb_assignment;
        else
            1;
        end

    end
    tpc = data(Nrbg+2:Nrbg+3); % 2 bits
    harq = data(Nrbg+4:Nrbg+6); % 3 bits
    tbcw_flag = data(Nrbg+7); % 1 bit
    tb1_mcs = data(Nrbg+8:Nrbg+12); % 5 bits
    tb1_ndi = data(Nrbg+13); % 1 bit
    tb1_rv = data(Nrbg+14:Nrbg+15); % 2 bits
    tb2_mcs = data(Nrbg+16:Nrbg+20); % 5 bits
    tb2_ndi = data(Nrbg+21); % 1 bit
    tb2_rv = data(Nrbg+22:Nrbg+23); % 2 bits    
    crnti = data(end-15:end); % 16 bits
    crnti_code = bin2dec(int2str(crnti.'));
    rb_code = bin2dec(int2str(rb_assignment.'));
    nscdl = sum(rb_assignment)*12*P;
    
    last_start = 0;    
    cur_end = 0;
    sclist = [];
    for ii=1:length(rb_assignment)
        if(rb_assignment(ii)==1)
            if(last_start == 0)
                last_start = -300+36*(ii-1);
                cur_end = -300+36*ii-1;
            else
                cur_end = -300+36*ii-1;                
            end
        else
            if(last_start ~= 0)                
                sclist = [sclist last_start cur_end];
                last_start = 0;
                cur_end = 0;
            end
        end
    end
    
    if(last_start ~= 0)
        sclist = [sclist last_start cur_end];        
    end
    sclist(sclist>=0) = sclist(sclist>=0)+1;
    
    fprintf(' Resource Alloc Hdr = %d\n RB-Assignment = [%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d]\n TPC Command = [%d %d]\n HARQ = [%d %d %d]\n TBCW_flag = %d\n TB1-MCS = [%d %d %d %d %d]\n TB1-NDI = %d\n TB1-RV = [%d %d]\n TB2-MCS = [%d %d %d %d %d]\n TB2-NDI = %d\n TB2-RV = [%d %d]\n CRNTI = [%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d] = %d\n', resource_alloc_hdr, rb_assignment, tpc, harq, tbcw_flag, tb1_mcs, tb1_ndi, tb1_rv, tb2_mcs, tb2_ndi, tb2_rv, crnti, crnti_code);    
    fprintf(' Actual subcarriers meant for CRNTI=%d on downlink: ', crnti_code);
    fprintf('[%d -> %d] ', sclist);
    fprintf('\n');
    
end