%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vit, succ] = decode_conv_lte_soft(d0, d1, d2, PRINT_DEBUG)

if(nargin <= 3)
    PRINT_DEBUG = 1;
end

len = length(d0);

conv_mat = [d0 d1 d2];

% Trellis preparations
constlen=7;
codegen=[133 171 165];
trellis=poly2trellis(constlen,codegen);

% Reshape input matrix for input to trellis decoding
conv_vec=reshape(conv_mat',[],1);

% Core idea: Repeat the conv vector twice to ensure correct state
%vit=vitdec([conv_vec;conv_vec],trellis,constlen,'trunc','soft',3);
vit = fast_vitdec([conv_vec;conv_vec]);
vit=vit(len+1:end);    

% Sanity Check
conv_sanity = fast_convenc([vit', vit']);
%conv_sanity = convenc([vit', vit'], trellis);
conv_sanity = (conv_sanity(3*len+1:end).')*7;

indx=find(conv_vec==0 | conv_vec==7);

test_conv_vec=conv_vec(indx);
test_conv_sanity = conv_sanity(indx);

if (sum(test_conv_sanity == test_conv_vec) == length(indx))
    if(PRINT_DEBUG) cprintf('blue', 'Conv decoder succeeded!\n'); end;
    succ = 1;    
elseif(sum(test_conv_sanity == test_conv_vec) >= length(indx)-4 || sum(test_conv_sanity == test_conv_vec) >= length(indx)*0.9)
    if(PRINT_DEBUG) cprintf('magenta', 'Convolutional decoder found SOME bit errors %d ~= %d\n', sum(test_conv_sanity == test_conv_vec), length(indx)); end;
    succ = 1;
else
    if(PRINT_DEBUG) fprintf(2, 'decode_conv_lte: Convolutional decoder found bit errors %d ~= %d\n', sum(test_conv_sanity == test_conv_vec), length(indx)); end;
    succ = 0;
end

end
