%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [avgchannel, offset] = get_corrected_channel_4port(x0, frame_num, slot_num, symbol_num, jump, subframe0_starts, is_extended_cp, is_zero_slope)
    nfft_used = 600;
    nfft = 1024;
    nport = 4;    
    nslot = 2;
    slot_len = 7680;    
    
    if(nargin < 7)
        is_extended_cp = false;
    end
    if(nargin < 8)
        is_zero_slope = false;
    end
    
    if(is_extended_cp)
        ncp0 = 256;
        ncp1 = 256;        
    else
        ncp0 = 80;
        ncp1 = 72;  
        
    end
    
    sep_l0l4 = (nfft+ncp0) + 3*(nfft+ncp1);
    sep_l0l1 = [(nfft+ncp0) , slot_len+(nfft+ncp0)];
    
    if((symbol_num == 0 && jump > ncp0) || (symbol_num > 0 && jump > ncp1) || jump < 0)
        fprintf(2, 'Warning: jump is much bigger than CP (or negative)\n');
    end
    
    offset = subframe0_starts(frame_num+1) + slot_num * slot_len + (symbol_num>0)*(nfft + ncp0) + max(0,symbol_num-1)*(nfft + ncp1) + jump - (symbol_num>0)*(ncp0 - ncp1);
    
    if(symbol_num == 0)
        jump0 = jump;
    else
        %jump0 = jump + ncp0 - ncp1;        
        jump0 = jump;
    end
    
    channel = NaN * ones(nfft, nslot, nport);
    
    avgchannel = NaN * ones(nfft, nport);

    [subframe_start, subframe_num] = get_closest_subframe_start(offset, subframe0_starts);
    
    sum_corr = 0;
    mean_corr = 0;
        
    for port = 1:min(2, nport)                    
        for slot = 1:nslot
            glob_slot = 2*subframe_num+slot-1;            
            
            % Symbol 0
            st = subframe_start+(slot-1)*slot_len + jump0;
            refs_fd_l0 = gen_fd_downlink_refsignal_accurate(glob_slot, 0, port-1);   
            refs_fft_l0 = fft_checkslope(x0(st:st+nfft-1), is_zero_slope, jump).';
            channel(~isnan(refs_fd_l0), slot, port) = refs_fft_l0(~isnan(refs_fd_l0))./refs_fd_l0(~isnan(refs_fd_l0));
            
            % Symbol 4 
            st = subframe_start+(slot-1)*slot_len + 4*(nfft+ncp1) + jump0;
            refs_fd_l4 = gen_fd_downlink_refsignal_accurate(glob_slot, 4, port-1);   
            refs_fft_l4 = fft_checkslope(x0(st:st+nfft-1), is_zero_slope, jump).';
            channel(~isnan(refs_fd_l4), slot, port) = refs_fft_l4(~isnan(refs_fd_l4))./refs_fd_l4(~isnan(refs_fd_l4));
        end        
        corr_l0=(angle(mean(channel(~isnan(refs_fd_l0), 2, port)./channel(~isnan(refs_fd_l0), 1, port))));
        channel(~isnan(refs_fd_l0), 2, port) = channel(~isnan(refs_fd_l0), 2, port) * exp(-1i*corr_l0);        
        avgchannel(~isnan(refs_fd_l0), port) = (channel(~isnan(refs_fd_l0), 1, port) + channel(~isnan(refs_fd_l0), 2, port))/2;
        
        corr_l4=(angle(mean(channel(~isnan(refs_fd_l4), 2, port)./channel(~isnan(refs_fd_l4), 1, port))));
        channel(~isnan(refs_fd_l4), 2, port) = channel(~isnan(refs_fd_l4), 2, port) * exp(-1i*corr_l4);        
        avgchannel(~isnan(refs_fd_l4), port) = (channel(~isnan(refs_fd_l4), 1, port) + channel(~isnan(refs_fd_l4), 2, port))/2;
        
        corr = (corr_l0+corr_l4)/2;
        avgchannel(~isnan(refs_fd_l4), port) = avgchannel(~isnan(refs_fd_l4), port) * exp(-1i* corr* sep_l0l4 / slot_len);
        
        %avgchannel(:, port) = avgchannel(:, port) * exp(1i *  corr * (offset-subframe_start) / slot_len);        
        avgchannel(:, port) = interpolate_nans(avgchannel(:, port));
        
        sum_corr = sum_corr + corr;
    end        
    
    mean_corr = sum_corr / min(2, nport);
    
    for port = 3:nport
        % Fill in avgchannel
        for slot = 1:nslot
            glob_slot = 2*subframe_num+slot-1;
            
            % Symbol 1
            st = subframe_start+(slot-1)*slot_len + (nfft+ncp1) + jump0;
            refs_fd_l1 = gen_fd_downlink_refsignal_accurate(glob_slot, 1, port-1);   
            refs_fft_l1 = fft_checkslope(x0(st:st+nfft-1), is_zero_slope, jump).';
            avgchannel(~isnan(refs_fd_l1), port) = refs_fft_l1(~isnan(refs_fd_l1))./refs_fd_l1(~isnan(refs_fd_l1));
                        
            % Correct for CFO
            avgchannel(~isnan(refs_fd_l1), port) = avgchannel(~isnan(refs_fd_l1), port) * exp(-1i* mean_corr* sep_l0l1(slot) / slot_len);
        end                
        
        avgchannel(:, port) = avgchannel(:, port) * exp(1i *  corr * (offset-subframe_start) / slot_len);        
        avgchannel(:, port) = interpolate_nans(avgchannel(:, port));
    end
    
    
end
