%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET_FRAME_BOUNDARIES_FAST Return subframe boundary indices
% Requires: Raw I/Q samples, Cell ID parameters m0, m1, nid0
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [subframe0_starts, subframe5_starts] = get_frame_boundaries_fast(x0, m0, m1, nid0)    
    nfft = 1024;
    ncp0 = 80;
    ncp1 = 72;
    slot_len = 7680;
    
    tdSSS_0=get_tdsss_m0_m1(m0, m1, nid0);    
    subframe0_starts = get_intelligent_peaks(tdSSS_0, x0) + (nfft+ncp1)-slot_len;
    tdSSS_1=get_tdsss_m0_m1(m1, m0, nid0);
    subframe5_starts = get_intelligent_peaks(tdSSS_1, x0) + (nfft+ncp1)-slot_len;
    
end

function starts = get_intelligent_peaks(tdSSS, x0)
    frame_len = 153600;
    future_thresh = 500;
    starts0 = get_peak_indices(abs(conv(tdSSS, x0(1:frame_len))));
    starts = starts0;
    nextstart = starts(end) + frame_len;
    while nextstart < length(x0)
        newstart = reshape(get_peak_indices(abs(conv(tdSSS, x0(nextstart-future_thresh:nextstart+future_thresh))))+(nextstart-future_thresh), 1, []);
        starts = [starts newstart];
        nextstart = starts(end) + frame_len;
    end
end

function subframeout = reinterpolate_subframe(subframein)
    subframeout=[];
    for ii=2:length(subframein)
        sdiff = subframein(ii)-subframein(ii-1);
        nextra = round(sdiff/153600)-1; 
        subframeout = [subframeout subframein(ii-1) + (0:nextra)*153600];
    end
    subframeout = [subframeout subframein(end)];
end
