%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET_PBCH_DATA Returns bits of PBCH in given subframe
% Requires: Raw I/Q samples, Frame number, Subframe number, # of base
% station antennas
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pbch_bits] =  get_pbch_data(x0, x1, frame_num, subframe0_starts, ntx_ant)
    nfft = 1024;    
    cell_params;        
    nfft_used = 300;    
    C_init = Ncellid;
    blen = 1920;    
    
    if(ntx_ant == 1)
        ant_mask = zeros(16,1);
    elseif(ntx_ant == 2)
        ant_mask = ones(16,1);
    end
    
    slot_num = 1;    
    npbch = 72;     
    
    pbch_subcarr = (nfft_used-npbch/2:nfft_used+npbch/2-1);
    pbch_subcarr = -300 + pbch_subcarr;
    pbch_subcarr(pbch_subcarr>=0) = pbch_subcarr(pbch_subcarr>=0) + 1;                
    
    bcnt = 0;    
    pbch_bits = zeros(blen, 1);
    
    for fn = frame_num:(frame_num+4-1)
        for sym = 0:3
            % Symbol 0 ... 3        
            refs_fd_indices_all = [];
            for ant=0:3
                refs_fd = gen_fd_downlink_refsignal_nowarn(slot_num, sym, ant);
                refs_fd_indices = sort(convert_bin_index_fft_to_normal(find(~isnan(refs_fd)), nfft)).';
                refs_fd_indices_all = reshape(sort(union(refs_fd_indices, refs_fd_indices_all)), 1, []);
            end
            pbch_subcarr_sym = setdiff(pbch_subcarr, refs_fd_indices_all);         


            pbch_data = invert_channel(x0, x1, fn, slot_num, sym, pbch_subcarr_sym, subframe0_starts);

            bits = demap_to_bits(pbch_data, length(pbch_subcarr_sym));
            pbch_bits(bcnt+1:bcnt+length(bits)) = bits;
            bcnt = bcnt + length(bits);
        end            
    end
    
    % Scramble
    c = get_c(C_init, blen);    
    pbch_bits = xor(c, pbch_bits);        
    
    % Interleave        
    [d0, d1, d2] = interleave_lte(pbch_bits);
    
    pbch_bits = decode_conv_lte_soft(d0, d1, d2);    
    
    
    % Test CRC (Note: This changes if the number of antennas change    
    crc_bits = crc16(pbch_bits(1:24));
    if(sum(xor(crc_bits(25:end), ant_mask) ~= pbch_bits(25:end)) > 0)
        fprintf(2, 'CRC failed\n');
    else
        fprintf(1, 'CRC succeeded!\n');
    end
end
    
    
