%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET_PCFIC_INFO_SISO Returns CFI number from PCFIC
% Requires: Raw I/Q samples, Frame number, Subframe number, Subframe start
% locations
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [descram_bits, cfi, snr] =  get_pcfic_info_siso(x0, frame_num, subframe_num, subframe0_starts)
    nfft = 1024;
    nscrb = 12;
    nrbdl = 50;
    cell_params;
    kbar = nscrb/2 * mod(Ncellid, 2*nrbdl); % function of cell ID (which is 0!)    
    nfft_used = 300;
    nsc_rg_max = 6;
    kbar_skip = floor(nrbdl/2)*(nscrb/2);
    
    pcfic_sc = (mod([ kbar+(-nfft_used: -nfft_used + nsc_rg_max -1), kbar+(-nfft_used + kbar_skip: -nfft_used + nsc_rg_max  + kbar_skip -1), ...
        kbar+(-nfft_used + 2*kbar_skip: -nfft_used + nsc_rg_max  + 2*kbar_skip -1), kbar+(-nfft_used + 3*kbar_skip: -nfft_used + nsc_rg_max  + 3*kbar_skip -1)]+nfft_used, 2*nfft_used)-nfft_used);        
    
    pcfic_sc(pcfic_sc>=0) = pcfic_sc(pcfic_sc>=0) +1;
            
    % Ignore reference signals
    pcfic_ex = [-nfft_used:-1, 1:nfft_used]; 
    pcfic_ex = pcfic_ex(mod(Ncellid, 3)+1:3:end);
    
    pcfic_sc = setdiff_orderpreserve(pcfic_sc, pcfic_ex);
    
    jump = 40;
    
    [avgchannel0, offset] = get_corrected_channel_4port(x0, frame_num, 2*subframe_num, 0, jump, subframe0_starts);
    %fplot(abs(avgchannel0(:,1)));  
    x0_fft = fft(x0(offset:offset+nfft-1));
            
    pcfic_data = zeros(16, 2);
    ii = 1;    
    
    sanity_check = 0;
    for ii=1:8
        scfft0 = convert_bin_index_normal_to_fft(pcfic_sc(2*ii-1), nfft);
        scfft1 = convert_bin_index_normal_to_fft(pcfic_sc(2*ii), nfft);
        hmat = [avgchannel0(scfft0, 1) avgchannel0(scfft0, 2); conj(avgchannel0(scfft1, 2)) -conj(avgchannel0(scfft1, 1))];        
        y = [x0_fft(scfft0); conj(x0_fft(scfft1))];
        x = (inv(hmat) * y).';
        pcfic_data(2*ii-1, :) = x;
        pcfic_data(2*ii, :)   = [-conj(x(2)) conj(x(1))];
        
        if(sign(real(pcfic_data(2*ii-1, 1))) == sign(real(pcfic_data(2*ii, 2))) && sign(imag(pcfic_data(2*ii-1, 1))) ~= sign(imag(pcfic_data(2*ii, 2))) && ...
           sign(real(pcfic_data(2*ii-1, 2))) ~= sign(real(pcfic_data(2*ii, 1))) && sign(imag(pcfic_data(2*ii-1, 2))) == sign(imag(pcfic_data(2*ii, 1))))
            sanity_check = sanity_check + 1;
        end
    end
    
    %figure; plot(pcfic_data, 'bx');
    
    % De-map the bits from QPSK
    bits = zeros(32, 1);
    for ii=1:8
       bits(4*ii-3) = sign(real(pcfic_data(2*ii-1, 1)) + real(pcfic_data(2*ii, 2))); % 0
       bits(4*ii-2) = sign(imag(pcfic_data(2*ii-1, 1)) - imag(pcfic_data(2*ii, 2))); % 1
       bits(4*ii-1) = sign(real(pcfic_data(2*ii, 1)) - real(pcfic_data(2*ii-1, 2))); % 2
       bits(4*ii)   = sign(imag(pcfic_data(2*ii-1, 2)) + imag(pcfic_data(2*ii, 1))); % 3
    end
    
    % Calculate SNR
    remap_bits = remap_data(bits);
    pcfic_sc_fft = convert_bin_index_normal_to_fft(pcfic_sc, nfft);
    y_actual = [x0_fft(pcfic_sc_fft).'];
    ii = 1;
    for sc = pcfic_sc
        scfft = convert_bin_index_normal_to_fft(sc, nfft);
        hmat = [avgchannel0(scfft, 1) avgchannel0(scfft, 2)];        
        y_pred(ii, :) = hmat * remap_bits(ii, :).';
        ii = ii + 1;
    end
    
    signal = y_actual;    
    
    noise = y_pred-y_actual;
    
    snr = 10*log10(mean(abs(signal(:)).^2)./mean(abs(noise(:)).^2));
    fprintf('SNR = %f\n', snr);

    
    bits = (1-bits)/2;
    
    % Now do the scrambling
    n_s = 2*subframe_num;
    C_init = (floor(n_s/2) + 1)*(2*Ncellid+1)*(2^9) + Ncellid;
    c = get_c(C_init, 32);        
    
    descram_bits = xor(c, bits);
    
    descram_0 = [0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1]';
    descram_1 = [1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0]';
    descram_2 = [1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1 0 1 1]';
    len = length(descram_0);
    
    if(sum(descram_bits == descram_0) == len)
        cfi = 1;
    elseif(sum(descram_bits == descram_1) == len)
        cfi = 2;
    elseif((sum(descram_bits == descram_2) == len))
        cfi = 3;
    else
        cfi = NaN;
        fprintf(2, 'Warning: CFI does not match known patterns!\n');
    end
    
    
    
    %figure(1); hold on; plot(pcfic_sc, abs(x0_pcfic), 'bo');
    %figure(2); hold on; plot(pcfic_sc, abs(x1_pcfic), 'ko');
    
end

function pcfic_data = remap_data(bits)    
    pcfic_data = zeros(length(bits)/2, 2);
    factor = 0.5;
    for ii=1:length(bits)/4
        pcfic_data(2*ii-1, 1) = ( bits(4*ii-3) + 1i*bits(4*ii-2) )*factor;
        pcfic_data(2*ii-1, 2) = (-bits(4*ii-1) + 1i*bits(4*ii) )*factor;
        pcfic_data(2*ii, 1)   = ( bits(4*ii-1) + 1i*bits(4*ii) )*factor;
        pcfic_data(2*ii, 2)   = ( bits(4*ii-3) - 1i*bits(4*ii-2) )*factor;
    end    
end
