%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [res, succ, y0, offset] = get_pdcch_blind_anysym_fast(x0, x1, frame_num, subframe_num, subframe0_starts,blen, plot_flag, pdch_indices)
    pow_thresh = 1;
    jump = 40;    
    slot_num = 2*subframe_num;
    nfft = 1024;
    sym = 0;
    nfft_used = 300;
    fft_thresh = 0.5; %0.045;
    allsc = [-nfft_used:-1, 1:nfft_used];
    cell_params;
    
    offset = get_offset_4port(frame_num, slot_num, sym, jump, subframe0_starts);

    x0_fft(:, 1) = fft(x0(offset:offset+1024-1));                                   
    
    [~, cfi] =  get_pcfic_info_siso(x0, frame_num, subframe_num, subframe0_starts);          

    if(isnan(cfi))
        res=[];
        succ = false;
        y0=[];
        return;
    end     
    
    pdch_indices_in = pdch_indices;
    
    if(cfi == 1)
        Nreg = floor(length(pdch_indices)/4);
        pdch_indices = [pdch_indices; ones(1, length(pdch_indices))].';
    elseif(cfi == 2)
        Nreg = floor((length(pdch_indices) + (cfi-1)*length(allsc))/4);        
        pdch_indices = sort_lm([pdch_indices.', ones(length(pdch_indices), 1)], [allsc.', 2*ones(length(allsc), 1)], Ncellid);                                
        offset1 = get_offset_4port(frame_num, slot_num, sym+1, jump, subframe0_starts);        
        x0_fft(:, 2) = fft(x0(offset1:offset1+1024-1));    
    elseif(cfi == 3)
        Nreg = floor((length(pdch_indices) + (cfi-1)*length(allsc))/4);
        pdch_indices = sort_lm(sort_lm([pdch_indices.', ones(length(pdch_indices), 1)], [allsc.', 2*ones(length(allsc), 1)], Ncellid), [allsc.', 3*ones(length(allsc), 1)], Ncellid);
        offset1 = get_offset_4port(frame_num, slot_num, sym+1, jump, subframe0_starts);        
        x0_fft(:, 2) = fft(x0(offset1:offset1+1024-1));    
        offset2 = get_offset_4port(frame_num, slot_num, sym+2, jump, subframe0_starts);        
        x0_fft(:, 3) = fft(x0(offset2:offset2+1024-1));    
    end
        
    if(plot_flag)                
        figure; hold on;
        plot(pdch_indices_in, abs(x0_fft(convert_bin_index_normal_to_fft(pdch_indices_in, nfft), 1)), 'bx');    
        if(cfi >= 2) 
            plot(allsc, abs(x0_fft(convert_bin_index_normal_to_fft(allsc, nfft), 2)), 'gx');    
        elseif(cfi == 3)
            plot(allsc, abs(x0_fft(convert_bin_index_normal_to_fft(allsc, nfft), 3)), 'cx');    
        end
    end  
        
    Ncce = floor(Nreg/9);        
        
    % Remove the interleaving of the CCEs
    gen_cce_interleave_anysym;
    
    subcarr_order = 4*reshape(repmat(cces_order, 4, 1), 1, []) + reshape(repmat([1 2 3 4]', Nreg_actual, 1), 1, []);
                
    
    y0 = zeros(size(pdch_indices, 1), 1);    
    for ii = 1:size(pdch_indices, 1)
        y0(ii) = x0_fft(convert_bin_index_normal_to_fft(pdch_indices(ii, 1), nfft), pdch_indices(ii, 2));        
    end           
        
    y0 = y0(subcarr_order, :);
    pdch_indices = pdch_indices(subcarr_order, :);           
    
    sum_abs = sum(abs(y0)>0.45)/36;
        
    CCE_list = [];
    for CCE_num = 0:Ncce-1
        cce_indices = 36*CCE_num+1:36*(CCE_num+1);                
        mean_fft = mean(abs(y0(cce_indices)));        
        if(mean_fft > fft_thresh)
           CCE_list = [CCE_list CCE_num];            
           if(plot_flag)                              
               h = plot(pdch_indices(cce_indices, 1), abs(y0(cce_indices)), 'ro');         
               hold on; title(sprintf('CCE = %d', CCE_num));
               %pause(1);
               %delete(h);
           end
        end                       
    end  
    
    [res, succ] =  get_pdcch_infer_blen_cce_outer(x0, x1, frame_num, subframe_num, subframe0_starts,blen,CCE_list, pdch_indices);        
end

function [res, succ] =  get_pdcch_infer_blen_cce_outer(x0, x1, frame_num, subframe_num, subframe0_starts,blen,CCE_list, pdch_indices)
    nchunks = floor(max(CCE_list)/8) + 1;
    if(isempty(CCE_list))
        nchunks = 0;
    end
    
    res = {};
    succ = [];    
    for ii=1:nchunks
        n1 = (ii-1)*8;
        ccel = CCE_list((CCE_list>=n1) & (CCE_list<(n1+8)));        
        [res0, succ0] =  get_pdcch_infer_blen_cce(x0, x1, frame_num, subframe_num, subframe0_starts,blen, ccel, ii-1, pdch_indices);
        res = [res res0];
        succ = [succ succ0];
    end
end

function [res, succ] =  get_pdcch_infer_blen_cce(x0, x1, frame_num, subframe_num, subframe0_starts,blen,CCE_list, chunknum, pdch_indices)
    %% Try each power of two starting with the biggest    
    dones = zeros(length(CCE_list), 1);    
    kk = 0;
    pp = 8;    
    full_list = (1:pp);
    cce_bitmap = zeros(pp, 1);
    cce_bitmap(CCE_list+1-8*chunknum) = 1;      
    res = {};
    succ = [];
 %       cce_bitmap
    while (pp >= 1)        
        % How many pp sized pieces are in full_list?
        npieces = length(full_list)/pp;        
        % For each piece, try and decode
        for ii=0:npieces-1
           piece =  (ii*pp+1 : (ii+1)*pp); % The current "piece"           
           % Is this piece valid? 
           % a) Is it in CCE_list
           % b) Is it already "solved"?
           % Both are captured by cce_bitmap           
           if(sum(cce_bitmap(piece)) == pp)                        
               [r, s] = get_pdcch_infer_blen(x0, x1, frame_num, subframe_num, subframe0_starts,blen, 8*chunknum+piece(1)-1, pp, pdch_indices);               
               res{piece(1)} = r;               
               succ(piece(1)) = s;               
               if(s)
                   fprintf(2, 'Success frame=%d, subframe=%d, blen=%d, piece=%d, pp=%d\n', frame_num, subframe_num, blen, piece(1)-1, pp);                   
                   cce_bitmap(piece) = zeros(pp, 1);                   
               end
           end                      
        end
        
        pp = pp / 2;
    end                
    notemp = ~cellfun('isempty', res);
    res = res(notemp);    
    succ = succ(notemp);    
    %succ = (sum(cce_bitmap)==0);
    %succ = repmat(succ, 1, length(res));
    
    %if(~succ)
    %    cce_locs = (find(cce_bitmap)-1);
    %end
end
    
function [res, succ] = get_pdcch_infer_blen(x0, x1, frame_num, subframe_num, subframe0_starts,blen,cce, cce_len, pdch_indices)
    succ = false;
    ii = 1;
    while(~succ && ii <= length(blen))
        [res, succ] = get_pdcch_anysym_fast(x0, x1, frame_num, subframe_num, subframe0_starts,blen(ii),cce,cce_len, pdch_indices);
        ii=ii+1;
    end
end
