%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2014 Swarun Kumar, (swarun at mit.edu)
% This project is Licensed under the MIT License (see LICENSE.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function complex_data = invert_channel_siso_complex(x0, frame_num, slot_num, subframe0_starts, pdch_indices)
    jump = 40;
    nfft = 1024;
    
    [avgchannel0, offset0] = get_corrected_channel_4port(x0, frame_num, slot_num, 0, jump, subframe0_starts);
    [avgchannel1, offset1] = get_corrected_channel_4port(x0, frame_num, slot_num, 1, jump, subframe0_starts);
    [avgchannel2, offset2] = get_corrected_channel_4port(x0, frame_num, slot_num, 2, jump, subframe0_starts);
    
    x0_fft0 = fft(x0(offset0:offset0+nfft-1));
    x0_fft1 = fft(x0(offset1:offset1+nfft-1));
    x0_fft2 = fft(x0(offset2:offset2+nfft-1));
        
    nsc = size(pdch_indices, 1);
    complex_data = zeros(nsc, 2);  
    
    sanity_check = 0;

    for ii=1:(nsc/2)
        scfft0 = convert_bin_index_normal_to_fft(pdch_indices(2*ii-1, 1), nfft);
        scfft1 = convert_bin_index_normal_to_fft(pdch_indices(2*ii, 1), nfft);
        
        if(pdch_indices(2*ii-1, 2) == 1 && pdch_indices(2*ii, 2) == 1)
            hmat = [avgchannel0(scfft0, 1) avgchannel0(scfft0, 2); conj(avgchannel0(scfft1, 2)) -conj(avgchannel0(scfft1, 1))];        
            y = [x0_fft0(scfft0); conj(x0_fft0(scfft1))];
        elseif(pdch_indices(2*ii-1, 2) == 2 && pdch_indices(2*ii, 2) == 2)
            hmat = [avgchannel1(scfft0, 1) avgchannel1(scfft0, 2); conj(avgchannel1(scfft1, 2)) -conj(avgchannel1(scfft1, 1))];        
            y = [x0_fft1(scfft0); conj(x0_fft1(scfft1))];
        elseif(pdch_indices(2*ii-1, 2) == 3 && pdch_indices(2*ii, 2) == 3)
            hmat = [avgchannel2(scfft0, 1) avgchannel2(scfft0, 2); conj(avgchannel2(scfft1, 2)) -conj(avgchannel2(scfft1, 1))];        
            y = [x0_fft2(scfft0); conj(x0_fft2(scfft1))];
        else
            fprintf(2, 'Got unexpected pdch_indices entry');
            throw('Error');            
        end
        x = (inv(hmat) * y).';
        complex_data(2*ii-1, :) = x;
        complex_data(2*ii, :)   = [-conj(x(2)) conj(x(1))];
        
        if(sign(real(complex_data(2*ii-1, 1))) == sign(real(complex_data(2*ii, 2))) && sign(imag(complex_data(2*ii-1, 1))) ~= sign(imag(complex_data(2*ii, 2))) && ...
           sign(real(complex_data(2*ii-1, 2))) ~= sign(real(complex_data(2*ii, 1))) && sign(imag(complex_data(2*ii-1, 2))) == sign(imag(complex_data(2*ii, 1))))
            sanity_check = sanity_check + 1;
        end
        
    end                    
end
